#include "animations.h"

// Hero animations.
extern TFrame const cFramesHeroIdle      [] = { {DIR_LEFT, g_hero_0}, {DIR_LEFT, g_hero_1} };
extern TFrame const cFramesHeroFalling   [] = { {DIR_LEFT, g_hero_2}, {DIR_LEFT, g_hero_3} };
extern TFrame const cFramesHeroJumping   [] = { {DIR_LEFT, g_hero_4}, };
extern TFrame const cFramesHeroWalking   [] = { {DIR_LEFT, g_hero_5}, {DIR_LEFT, g_hero_6} };
extern TFrame const cFramesHeroUnsheating[] = { {DIR_LEFT, g_herofight_00}, {DIR_LEFT, g_herofight_01}, {DIR_LEFT, g_herofight_02} };
extern TFrame const cFramesHeroHolding   [] = { {DIR_LEFT, g_herofight_03}, {DIR_LEFT, g_herofight_04} };
extern TFrame const cFramesHeroAttacking [] = { {DIR_LEFT, g_herofight_05}, {DIR_LEFT, g_herofight_06}, {DIR_LEFT, g_herofight_07} };
extern TFrame const cFramesHeroDefending [] = { {DIR_LEFT, g_herofight_08}, {DIR_LEFT, g_herofight_09} };
extern TFrame const cFramesHeroHit       [] = { {DIR_LEFT, g_herofight_10}, {DIR_LEFT, g_herofight_11} };
extern TFrame const cFramesHeroStepping  [] = { {DIR_LEFT, g_herofight_12}, {DIR_LEFT, g_herofight_13} };
extern TFrame const cFramesHeroSheating  [] = { {DIR_LEFT, g_herofight_14}, {DIR_LEFT, g_herofight_15} };
extern TFrame const cFramesHeroDying     [] = { {DIR_LEFT, g_herofight_16}, {DIR_LEFT, g_herofight_17}, {DIR_LEFT, g_herofight_18} };

// Enemy animations.
extern TFrame const cFramesEnemyIdle      [] = { {DIR_LEFT, g_enemy_0}, {DIR_LEFT, g_enemy_1} };
extern TFrame const cFramesEnemyFalling   [] = { {DIR_LEFT, g_enemy_2}, {DIR_LEFT, g_enemy_3} };
extern TFrame const cFramesEnemyJumping   [] = { {DIR_LEFT, g_enemy_4}, };
extern TFrame const cFramesEnemyWalking   [] = { {DIR_LEFT, g_enemy_5}, {DIR_LEFT, g_enemy_6} };
extern TFrame const cFramesEnemyUnsheating[] = { {DIR_LEFT, g_enemyfight_00}, {DIR_LEFT, g_enemyfight_01}, {DIR_LEFT, g_enemyfight_02} };
extern TFrame const cFramesEnemyHolding   [] = { {DIR_LEFT, g_enemyfight_03}, {DIR_LEFT, g_enemyfight_04} };
extern TFrame const cFramesEnemyAttacking [] = { {DIR_LEFT, g_enemyfight_05}, {DIR_LEFT, g_enemyfight_06}, {DIR_LEFT, g_enemyfight_07} };
extern TFrame const cFramesEnemyDefending [] = { {DIR_LEFT, g_enemyfight_08}, {DIR_LEFT, g_enemyfight_09} };
extern TFrame const cFramesEnemyHit       [] = { {DIR_LEFT, g_enemyfight_10}, {DIR_LEFT, g_enemyfight_11} };
extern TFrame const cFramesEnemyStepping  [] = { {DIR_LEFT, g_enemyfight_12}, {DIR_LEFT, g_enemyfight_13} };
extern TFrame const cFramesEnemySheating  [] = { {DIR_LEFT, g_enemyfight_14}, {DIR_LEFT, g_enemyfight_15} };
extern TFrame const cFramesEnemyDying     [] = { {DIR_LEFT, g_enemyfight_16}, {DIR_LEFT, g_enemyfight_17}, {DIR_LEFT, g_enemyfight_18} };

// Table with animation data: number of frames, frame delay and frame array pointer for each character state.
const TAnimData const cCharacterAnimations[2][NUM_CHAR_STATES] = {
   {
      // Hero.
      {2, FRAMEDELAY_MEDIUM,   cFramesHeroIdle},
      {2, FRAMEDELAY_SHORT-2,  cFramesHeroWalking},
      {2, FRAMEDELAY_MEDIUM,   cFramesHeroJumping},
      {2, FRAMEDELAY_MEDIUM,   cFramesHeroFalling},
      ANIM_NULL,
      {3, FRAMEDELAY_MEDIUM,   cFramesHeroUnsheating},
      {3, FRAMEDELAY_SHORT+2,  cFramesHeroAttacking},
      {2, FRAMEDELAY_MEDIUM-3, cFramesHeroHolding},
      {2, FRAMEDELAY_SHORT,    cFramesHeroStepping},
      {2, FRAMEDELAY_MEDIUM,   cFramesHeroDefending},
      {2, FRAMEDELAY_LONG,     cFramesHeroSheating},
      {2, FRAMEDELAY_SHORT+2,  cFramesHeroHit},
      {3, FRAMEDELAY_MEDIUM,   cFramesHeroDying},
      ANIM_NULL,
   },
   {
      // Enemy.
      {2, FRAMEDELAY_MEDIUM,   cFramesEnemyIdle},
      {2, FRAMEDELAY_SHORT-2,  cFramesEnemyWalking},
      {2, FRAMEDELAY_MEDIUM,   cFramesEnemyJumping},
      {2, FRAMEDELAY_MEDIUM,   cFramesEnemyFalling},
      ANIM_NULL,
      {3, FRAMEDELAY_MEDIUM,   cFramesEnemyUnsheating},
      {3, FRAMEDELAY_SHORT+2,  cFramesEnemyAttacking},
      {2, FRAMEDELAY_MEDIUM-2, cFramesEnemyHolding},
      {2, FRAMEDELAY_SHORT,    cFramesEnemyStepping},
      {2, FRAMEDELAY_MEDIUM,   cFramesEnemyDefending},
      {2, FRAMEDELAY_LONG,     cFramesEnemySheating},
      {2, FRAMEDELAY_SHORT+2,  cFramesEnemyHit},
      {3, FRAMEDELAY_MEDIUM,   cFramesEnemyDying},
      ANIM_NULL,
   },
};


void initCharacterAnimation(TCharacter *c) {
   c->animation.frameCounter = 0;
   c->animation.redrawState = REDRAW_NO;
   if (c == &hero) {
      c->animation.pFrame = &cFramesHeroIdle[0];
      c->animation.width = G_HERO_0_W;
      c->animation.height = G_HERO_0_H;
   }
   else {
      c->animation.pFrame = &cFramesEnemyIdle[0];
      c->animation.width = G_ENEMY_0_W;
      c->animation.height = G_ENEMY_0_H;
   }
}


/**
 * Updates animation (counter & pointer) and also character state for special animations.
 *
 * A more sophisticated solution would be to throw all of the frame sequences, lenghts etc. in
 * data structures to make the animation system more flexible for multiple entities.
 * But time's critical…
 * @param c
 * @param i 0 if it's the player animation, 1 for enemies
 */
void updateCharAnimation(TCharacter *c) {
   TAnimation *ani = &c->animation;
   u8 *state = &c->state, type = !(c == &hero);
   TAnimData *aniData = &cCharacterAnimations[type][*state];
   TFrame *oldFrame = ani->pFrame;
   u8 animFinished = ++ani->frameCounter == aniData->numFrames * aniData->delay;
   // Reset frame counter?
   if (animFinished) {
      ani->frameCounter = 0;
      // Transition to another state as needed when animation ends.
      switch (*state) {
         case ST_FIGHT_STEPPING:
            *state = ST_FIGHT_HOLDING;
            break;
         case ST_FIGHT_ATTACKING:
            *state = ST_FIGHT_HOLDING;
            break;
         case ST_FIGHT_DEFENDING:
            *state = ST_FIGHT_HOLDING;
            break;
         case ST_FIGHT_HIT:
            *state = ST_FIGHT_HOLDING;
            break;
         case ST_FIGHT_UNSHEATING:
            *state = ST_FIGHT_HOLDING;
            break;
         case ST_FIGHT_SHEATING:
            leaveFight(c);
            break;
         case ST_DYING:
            *state = ST_DEAD;
            return;     // Don't change the frame anymore.
            break;
      }
      aniData = &cCharacterAnimations[type][*state];
   }
   // Set correct frame of the sequence.
   ani->pFrame = aniData->firstFrame + (ani->frameCounter / aniData->delay);
   if (oldFrame != ani->pFrame && ani->redrawState == REDRAW_NO) {
      // Frame changed and we're not leaving a fight (changing size).
      ani->redrawState = REDRAW_YES;
   }
}