#ifndef ENTITIES_H
#define ENTITIES_H

#include <types.h>
#include "globals.h"
#include "screens.h"
#include "characters.h"
#include "animations.h"
#include "ai.h"
#include "gamemode/game.h"
#include "utils/utils.h"
#include "assets/coin.h"
#include "assets/finish.h"
#include "assets/key_r.h"
#include "assets/key_g.h"
#include "assets/key_b.h"
#include "assets/key_y.h"
#include "assets/lock_r.h"
#include "assets/lock_g.h"
#include "assets/lock_b.h"
#include "assets/lock_y.h"
#include "assets/hero.h"
#include "assets/herofight.h"
#include "assets/enemy.h"
#include "assets/enemyfight.h"

///////////////////////////////////////////////////////////////////////////////
// Variables.
//

extern TCharacter allEnemies[MAX_ENEMIES];
extern TCoin      allCoins[MAX_COINS];
extern TKey       allKeys[MAX_KEYS];
extern TLock      allLocks[MAX_LOCKS];
extern TBridge    allBridges[MAX_BRIDGES];
extern TCharacter hero;
extern TFinish    finish;

extern u8 numCoins, numKeys, numLocks, numBridges, numEnemies;
extern EKeyColor keyInventory;
extern u8 numCollectedCoins;

///////////////////////////////////////////////////////////////////////////////
// Functions.
//

void initEntities();
void updateEntities();

void spawnCoin(u8 posX, u8 posY);
void spawnFinish(u8 posX, u8 posY);
void spawnKey(u8 posX, u8 posY, EKeyColor color);
void spawnLock(u8 posX, u8 posY, EKeyColor color);
void spawnBridge(u8 tileX, u8 tileY, TScreen *scr, EDirection dir);
void spawnPlayer(u8 posX, u8 posY);
void spawnEnemy(u8 posX, u8 posY);
void activateLock(TLock *l);
void extendBridge(TBridge *b, u8 isOnScreen);
void prepareBridgeOffscreen(TScreen *scr, EDirection dir, u8 tileY);
void setCharacterPosition(TCharacter *chr, u8 x, u8 y);
void keepInsideScreen();

#endif // ENTITIES_H
