#include "menu.h"

#define TITLE_X 80/2 - G_TITLETEXT_W / 2
#define TITLE_Y 4
#define TITLE_VMEM cpctm_screenPtr(CPCT_VMEM_START, TITLE_X, TITLE_Y)
#define MENU_AREA_X 20
#define MENU_AREA_Y 70
#define MENU_VMEM cpctm_screenPtr(CPCT_VMEM_START, MENU_AREA_X, MENU_AREA_Y)
#define MENU_AREA_WIDTH 60
#define MENU_AREA_HEIGHT 50
#define MENU_ENTRY_H G_FONT_BITMAPFONT_00_H + 3
#define TICKER_HEIGHT (G_FONT_BITMAPFONT_00_H+10)
#define TICKER_Y 199-TICKER_HEIGHT
#define TICKER_TEXTX 4
#define TICKER_VMEM_TOP    cpctm_screenPtr(CPCT_VMEM_START, 0, TICKER_Y)
#define TICKER_VMEM_CENTER cpctm_screenPtr(CPCT_VMEM_START, 0, TICKER_Y+1)
#define TICKER_VMEM_BOTTOM cpctm_screenPtr(CPCT_VMEM_START, 0, TICKER_Y+TICKER_HEIGHT)
#define TICKER_DELAY 250
#define TICKER_MESSAGE_COUNT 10

enum {
   OPTION_PLAY = 1,
   OPTION_REDEFINE,
   OPTION_TWOPLAYERS,
};

enum {
   STATE_MAIN,
   STATE_REDEFINE,
   STATE_PLAY
};

char* const cTickerMessages[TICKER_MESSAGE_COUNT] = {
   "A game by Norman",
   "For CPCRetroDev 2020",
   "Made using CPCTelera",
   "And Arkos Tracker 2",
   "Special thanks to:",
   "> Ronaldo (CPCTelera & support)",
   "> Targhan (Arkos Tracker 2)",
   "> Otto (advice & testing)",
   "> And Gryzor's cpcwiki.eu forum",
   "Deo confidimus!",
};


u8 currentState, nextState, tickerIndex;
u16 tickerTimer;


u8 getMenuKey() {
   static const cpct_keyID keys[4] = {Key_1, Key_2, Key_3, Key_4};
   return checkKeys(keys, 4);
}


cpct_keyID redefineKey(u8 *text){
   cpct_keyID key;
   cpct_waitVSYNC();
   drawText("Press key for              ", MENU_AREA_X+2, MENU_AREA_Y+15, 0);
   drawText(text, MENU_AREA_X+2+G_FONT_BITMAPFONT_00_W*14, MENU_AREA_Y+15, 0);
   key = waitForKeyPress();
   waitForKeyRelease(key);
   return key;
}


void clearMenuArea() {
   cpct_drawSolidBox(MENU_VMEM, PATTERN_BLACK, MENU_AREA_WIDTH, MENU_AREA_HEIGHT);
}


void renderMainMenu() {
   u8 y = MENU_AREA_Y;
   cpct_waitVSYNC();
   clearMenuArea();
   // Menu entries.
   drawText("1 Play",             MENU_AREA_X, y, 0);     y += MENU_ENTRY_H;
   drawText("2 Redefine Keys P1", MENU_AREA_X, y, 0);     y += MENU_ENTRY_H;
   drawText("3 Redefine Keys P2", MENU_AREA_X, y, 0);     y += MENU_ENTRY_H;
   cpct_drawSolidBox(cpct_getScreenPtr(CPCT_VMEM_START, MENU_AREA_X, y-7), PATTERN_RED, 18*G_FONT_BITMAPFONT_00_W, 1);
   drawText("4 Two players",      MENU_AREA_X, y, 0);     y += MENU_ENTRY_H;
   cpct_drawSolidBox(cpct_getScreenPtr(CPCT_VMEM_START, MENU_AREA_X, y-7), PATTERN_RED, 13*G_FONT_BITMAPFONT_00_W, 1);
}


void renderRedefine() {
   clearMenuArea();
   while(cpct_isAnyKeyPressed_f());    // Wait until all keys are released before defining.
   definedKeys[0] = redefineKey("LEFT");
   definedKeys[1] = redefineKey("RIGHT");
   definedKeys[2] = redefineKey("UP/ATTACK");
   definedKeys[3] = redefineKey("DOWN/DEFEND");
   definedKeys[4] = redefineKey("USE");
}


void renderInfoTicker() {
   tickerTimer++;
   if (tickerTimer >= TICKER_DELAY) {
      tickerTimer = 0;
      cpct_drawSolidBox(cpct_getScreenPtr(TICKER_VMEM_CENTER, 0,  0), PATTERN_DARKBLUE, SCREEN_WIDTH/2, TICKER_HEIGHT-1);
      cpct_drawSolidBox(cpct_getScreenPtr(TICKER_VMEM_CENTER, 40, 0), PATTERN_DARKBLUE, SCREEN_WIDTH/2, TICKER_HEIGHT-1);
      drawText(cTickerMessages[tickerIndex], TICKER_TEXTX, TICKER_Y+6, 1);
      tickerIndex++;
      if (tickerIndex == TICKER_MESSAGE_COUNT) {
         tickerIndex = 0;
      }
   }
}


/**
 * Render static parts of the menu.
 */
void showMenu() {
   initSound();   // Start playing music.
   cpct_clearScreen_f64(PATTERN_BLACK);
   currentState = nextState = STATE_MAIN;
   tickerTimer = TICKER_DELAY;
   tickerIndex = 0;
   cpct_waitVSYNC();
   // Title and subscript.
   cpct_drawSprite(g_titletext, TITLE_VMEM, G_TITLETEXT_W, G_TITLETEXT_H);
   drawText("The Adventures of a not-so-lucky Knight", 0, TITLE_Y+G_TITLETEXT_H+8, 0);
   // Ticker outline.
   cpct_drawSolidBox(cpct_getScreenPtr(TICKER_VMEM_TOP,    0,  0), PATTERN_WHITE, SCREEN_WIDTH/2, 1);
   cpct_drawSolidBox(cpct_getScreenPtr(TICKER_VMEM_TOP,    40, 0), PATTERN_WHITE, SCREEN_WIDTH/2, 1);
   cpct_drawSolidBox(cpct_getScreenPtr(TICKER_VMEM_BOTTOM, 0,  0), PATTERN_WHITE, SCREEN_WIDTH/2, 1);
   cpct_drawSolidBox(cpct_getScreenPtr(TICKER_VMEM_BOTTOM, 40, 0), PATTERN_WHITE, SCREEN_WIDTH/2, 1);
   // Menu entries.
   renderMainMenu();
   while(currentState != STATE_PLAY) {
      switch(getMenuKey()) {
         case 1:
            currentState = STATE_PLAY;
            break;
         case 2:
            currentState = STATE_REDEFINE;
            renderRedefine();
            renderMainMenu();
         case 3:
            break;
         case 4:
            break;
      }
      if(cpct_isKeyPressed(Joy0_Fire1)) {
         currentState = STATE_PLAY;
      }
      cpct_waitVSYNC();
      renderInfoTicker();
   }
   stopMusic();
}