#include "keyboard.h"

cpct_keyID definedKeys[NUM_KEYS];


/**
 * Default key bindings.
 */
void initKeys() {
   definedKeys[0] = Joy0_Left;
   definedKeys[1] = Joy0_Right;
   definedKeys[2] = Joy0_Up;
   definedKeys[3] = Joy0_Down;
   definedKeys[4] = Joy0_Fire1;
}


u8 checkKeys(const cpct_keyID* k, u8 numk) {
   if (cpct_isAnyKeyPressed_f()) {
      for (u8 i = 1; i <= numk; i++, k++) {
         if (cpct_isKeyPressed(*k))
            return i;
      }
   }
   return 0;
}


/**
 * Function created by Ronaldo for redefining keys.
 */
cpct_keyID waitForKeyPress() {
   // Backwards loop to 0.
   u8 i = 10, *keys = cpct_keyboardStatusBuffer + 9;
   u16 keypressed;

   while (!cpct_isAnyKeyPressed_f());

   do {
      // If in this 80 keys group there is any key pressed, some of their bits will be 0.
      // Else, all values will be 1, and a XOR with 0xFF returns 0 (FALSE).
      keypressed = *keys ^ 0xFF;
      if (keypressed) {
         return (keypressed << 8) + (i - 1); // Format cpct_keyID: 8 first bits = key maks, the next 8 are the row in keyboard (0-9)
      }
      keys--;
   } while (--i);

   return 0;   // Satisfy SDCC.
}


void waitForKeyRelease(cpct_keyID key) {
   while (cpct_isKeyPressed(key));
}


/**
 * Wait until all keys are released, then wait until a key is pressed.
 */
void waitForNextKeyPress() {
   while (cpct_isAnyKeyPressed_f()) {      // Wait for release of all keys.
      cpct_waitVSYNC();
   }
   while (!cpct_isAnyKeyPressed_f()) {     // Wait for a keypress.
      cpct_waitVSYNC();
   }
}