#include "cpctelera.h"
#include "stdio.h"
#include "gamemode/game.h"
#include "gamemode/menu.h"
#include "gamemode/ending.h"
#include "assets/tiles.h"
#include "sound/sound.h"
#include "keyboard.h"

#define MESSAGE_X 0
#define MESSAGE_Y 70
#define MESSAGE_WIDTH 60
#define MESSAGE_HEIGHT 40
#define MESSAGE_CENTER MESSAGE_X + MESSAGE_WIDTH / 2
#define MESSAGE_VMEM cpctm_screenPtr(CPCT_VMEM_START, MESSAGE_X, MESSAGE_Y-3)
#define NUM_CHARACTERS (MESSAGE_WIDTH-2) / G_FONT_BITMAPFONT_00_W


void drawMessageStringCentered(char buffer[], u8 length) {
   drawText(buffer, MESSAGE_CENTER - length*G_FONT_BITMAPFONT_00_W/2, MESSAGE_Y, 0);
}


/**
 * Show a funny message related to how the player died.
 */
void showDeathMessage() {
   cpct_drawSolidBox(cpct_getScreenPtr(MESSAGE_VMEM, 0, 0),  PATTERN_BLACK, 40, MESSAGE_HEIGHT);
   cpct_drawSolidBox(cpct_getScreenPtr(MESSAGE_VMEM, 40, 0), PATTERN_BLACK, 40, MESSAGE_HEIGHT);
   drawText("You died!", 31, MESSAGE_Y, 1);
   if (gameState == GAME_PLAYER_KILLED) {    // Got stabbed.
      u8 choice = cpct_rand8() % 3;
      switch(choice) {
         case 0:
            drawText("Remember, just keep your health above 0",  0, MESSAGE_Y+15, 1);
            drawText("while lowering your enemy's health to 0.", 0, MESSAGE_Y+25, 1);
            break;
         case 1:
            drawText("It looks like you just got stabbed.", 7, MESSAGE_Y+15, 1);
            break;
         case 2:
            drawText("He struggled - and died.", 20, MESSAGE_Y+15, 1);
            break;
      }
   }
   else {      // Fell to death.
      u8 choice = cpct_rand8() % 3;
      switch (choice) {
         case 0:
            drawText("Tip: Falling out the bottom is", 10, MESSAGE_Y+15, 1);
            drawText("detrimental to Fips' health.",   10, MESSAGE_Y+25, 1);
            break;
         case 1:
            drawText("Tip: Don't try to leave",  15, MESSAGE_Y+15, 1);
            drawText("the level at the bottom.", 15, MESSAGE_Y+25, 1);
            break;
         case 2:
            drawText("You went a bit too far down.", 13, MESSAGE_Y+15, 1);
            break;
      }
   }
   waitForNextKeyPress();
}

/**
 * Interrupt runs at 300Hz. Sound should be played at 50Hz.
 */
void handleInterrupt(void) {
   static u8 counter = 0;
   if (counter == 0) {
      cpct_scanKeyboard_if();
   } else if (counter == 1) {
      playSound();
   }

   if (++counter == 6) {
      counter = 0;
   }
}


void initializeCPC() {
   cpct_disableFirmware();
   cpct_setPALColour(1, HW_BLACK);           // Fill screen
   cpct_clearScreen_f64(PATTERN_BLACK);      // with black.
   cpct_setPalette(g_palette, 16);
   cpct_setVideoMode(0);
   cpct_setBorder(HW_BLACK);
   cpct_etm_setTileset2x4(g_tileset);
   cpct_setInterruptHandler(handleInterrupt);
   initSound();
   initKeys();
}


/**
 * Function for one playthrough of the game.
 */
void gameCycle() {
   while (currentLevel < NUM_LEVELS) {
      cpct_clearScreen_f64(PATTERN_BLACK);
      drawText("Loading level...", 40-16, 76, 1);
      levelScore = 0;
      showGame();
      cpct_waitHalts(10);
      switch(gameState) {           // Evaluate how level ended.
         case GAME_PLAYER_KILLED:
         case GAME_PLAYER_FELL:
            showDeathMessage();
            break;
         case GAME_LEVEL_FINISHED:
            cpct_drawSolidBox(cpctm_screenPtr(CPCT_VMEM_START, 40-16, 76), PATTERN_BLACK, 32, 8);
            drawText("Level completed!", 40-16, 76, 1);
            currentLevel++;
            globalScore += levelScore;
            waitForNextKeyPress();
            break;
      }
      if (currentLevel == NUM_LEVELS) {
         showEnding();
         break;
      }
      if (gameState == GAME_BACK_TO_MENU) {
         break;
      }
   }
}


void main() {
   initializeCPC();
   currentLevel = 0;
   globalScore = 0;
   while (1) {
      showMenu();
      gameCycle();
   }
}