#include "render.h"

#define HUD_TOP_Y           4
#define LEVEL_DISPLAY_X     1
#define LEVEL_DISPLAY_Y     HUD_TOP_Y
#define SCORE_DISPLAY_X     25
#define SCORE_DISPLAY_Y     HUD_TOP_Y
#define SCORE_DISPLAY_SIZE  6*G_FONT_BITMAPFONT_00_W
#define KEY_DISPLAY_VMEM    cpctm_screenPtr(CPCT_VMEM_START, 30, 16)
#define KEY_OFFSET          (G_KEY_R_W+1)
#define COIN_DISPLAY_SIZE   6 * G_FONT_BITMAPFONT_00_W
#define COIN_DISPLAY_X      SCREEN_WIDTH - COIN_DISPLAY_SIZE - 2 * G_FONT_BITMAPFONT_00_W
#define COIN_DISPLAY_Y      HUD_TOP_Y
#define COIN_DISPLAY_VMEM   cpctm_screenPtr(CPCT_VMEM_START, COIN_DISPLAY_X, 2)
#define HEALTH_DISPLAY_VMEM cpctm_screenPtr(CPCT_VMEM_START, 1, 16)

#ifndef TABLETRANSP
   #define TABLETRANSP
   cpctm_createTransparentMaskTable(g_tabletransp, 0xA500, M0, 0);
#endif

EKeyColor keyDisplay;   // Stores state of the key inventory HUD.


/**
 * Inititializes the HUD, draws basic information.
 */
void initHud() {
   char txtLevel[] = "Level   ";
   char txtScore[] = "Score     ";
   keyDisplay = COLOR_NONE;

   u8ToString(&txtLevel[6], currentLevel+1);
   drawText(txtLevel, LEVEL_DISPLAY_X, LEVEL_DISPLAY_Y, 0);
   drawText(txtScore, SCORE_DISPLAY_X, SCORE_DISPLAY_Y, 0);
   drawText("Coins ", COIN_DISPLAY_X, COIN_DISPLAY_Y, 0);
   redrawHudFlags = 255;   // Redraw everything.
}


/**
 *
 * @param gamePosX
 * @param gamePosY
 * @param sprite
 */
void renderSprite(u8 gamePosX, u8 gamePosY, TSprite *sprite) {
   const u8 *vmem = cpct_getScreenPtr(CPCT_VMEM_START, gamePosX, gamePosY + MAP_POS_Y);
   cpct_drawSpriteMaskedAlignedTable(sprite->dataPtr, vmem, sprite->width, sprite->height, g_tabletransp);
}


/**
 * This function is inlined to save a few cycles in the "race against the beam" (VSync)
 * @param gamePosX
 * @param gamePosY
 * @param sprite
 */
inline void renderAnimFrame(TCharacter *c) {
   const TAnimation *ani = &c->animation;
   const u8 *vmem        = cpct_getScreenPtr(CPCT_VMEM_START, c->entity.x, c->entity.y + MAP_POS_Y);
   cpct_drawSpriteMaskedAlignedTable(ani->pFrame->dataPtr, vmem, ani->width, ani->height, g_tabletransp);
}


inline void renderCharacter(TCharacter *c) {
   TAnimation *ani  = &c->animation;
   TFrame *framePtr = ani->pFrame;
   if (framePtr->direction != c->facingDirection) {
      cpct_hflipSpriteM0(ani->width, ani->height, ani->pFrame->dataPtr);
      framePtr->direction = c->facingDirection;
   }
   renderAnimFrame(c);
   ani->redrawState = REDRAW_NO;
}


void eraseCharacter(TCharacter *chr) {
   const u8 prevX = chr->entity.px;
   const u8 prevY = chr->entity.py;
   const u8 aniWidth = chr->animation.width;
   // Calculate width: Ugly, but it works.
   //const u8 w = aniWidth <= 3 ? 2 : (3 + (prevX + aniWidth & 1 ? 1 : 0));
   const u8 w = aniWidth <= 3 ? 2 : (3 + (prevX + aniWidth & 1 ? 1 : 0));
   const u8 h = chr->animation.height / 4 + (prevY & 0b11 ? 1 : 0);
   cpct_etm_drawTileBox2x4(prevX/2, prevY/4, w, h, MAP_WIDTH, MAP_POS, currentScreenTiles);
   if (chr->animation.redrawState == REDRAW_LEAVEFIGHT) {
      // We're leaving a fight, change size and position of character.
      u8 offset = 0;
      if (chr == &hero) {
         offset = G_HEROFIGHT_00_W-G_HERO_0_W;
         hero.animation.width = G_HERO_0_W;
         hero.animation.height = G_HERO_0_H;
      }
      else {
         offset = G_ENEMYFIGHT_00_W-G_ENEMY_0_W;
         chr->animation.width = G_ENEMY_0_W;
         chr->animation.height = G_ENEMY_0_H;
      }
      if (chr->facingDirection == DIR_LEFT) {
         // Correct position when changing sprite size.
         setCharacterPosition(chr, chr->entity.x+offset, chr->entity.y);
      }
   }
   // Draw debug box.
   //cpct_drawSolidBox(cpct_getScreenPtr(CPCT_VMEM_START, gamePosX, MAP_POS_Y+gamePosY), (u16) (cpctm_px2byteM0(0, 0) << 8) + cpctm_px2byteM0(0, 0), w*2, h*4);
}


void eraseCoin(TCoin *c) __z88dk_fastcall {
   const u8 prevX = c->entity.x, gamePosY = c->entity.y;
   const u8 w = G_COIN_W / 2 + (prevX & 0b1);
   const u8 h = G_COIN_H / 4 + 1;
   cpct_etm_drawTileBox2x4(prevX/2, gamePosY/4, w, h, MAP_WIDTH, MAP_POS, currentScreenTiles);
}


void eraseKey(TKey *k) __z88dk_fastcall {
   const u8 prevX = k->entity.x, gamePosY = k->entity.y;
   const u8 w = G_KEY_R_W / 2 + (prevX & 0b1);
   const u8 h = G_KEY_R_H / 4 + 1;
   cpct_etm_drawTileBox2x4(prevX/2, gamePosY/4, w, h, MAP_WIDTH, MAP_POS, currentScreenTiles);
}


void updateKeyDisplay(EKeyColor color, u8* sprite, u8 offset) {
   u8 hasKey         = (keyInventory & color) == color,
      isKeyDisplayed = (keyDisplay & color) == color,
      *screenPtr     = cpct_getScreenPtr(KEY_DISPLAY_VMEM, offset*KEY_OFFSET, 0);
   if (hasKey && !isKeyDisplayed) {
      cpct_drawSpriteMaskedAlignedTable(sprite, screenPtr, G_KEY_R_W, G_KEY_R_H, g_tabletransp);
      keyDisplay |= color;
   }
   else if (!hasKey && isKeyDisplayed) {
      cpct_drawSolidBox(screenPtr, PATTERN_BLACK, G_KEY_R_W, G_KEY_R_H);
      keyDisplay &= ~color;
   }
}


void drawHud() {
   // Coins HUD.
   if ((redrawHudFlags & REDRAW_HUD_COINS) == REDRAW_HUD_COINS) {
      char buffer[3];
      u8ToString(buffer, numCollectedCoins);
      drawText(buffer, COIN_DISPLAY_X+COIN_DISPLAY_SIZE, COIN_DISPLAY_Y, 0);
   }
   // Keys HUD.
   if ((redrawHudFlags & REDRAW_HUD_KEYS) == REDRAW_HUD_KEYS) {
      updateKeyDisplay(COLOR_RED,    g_key_r, 0);
      updateKeyDisplay(COLOR_GREEN,  g_key_g, 1);
      updateKeyDisplay(COLOR_BLUE,   g_key_b, 2);
      updateKeyDisplay(COLOR_YELLOW, g_key_y, 3);
   }
   // Health HUD.
   if ((redrawHudFlags & REDRAW_HUD_HEROHEALTH) == REDRAW_HUD_HEROHEALTH) {
      for (i8 i = HEALTH_HERO_START-1; i >= 0; i--) {
         if ((u8)i < hero.health) {
            cpct_drawSpriteMaskedAlignedTable(g_healthhud_1,
               cpct_getScreenPtr(HEALTH_DISPLAY_VMEM, i*(G_HEALTHHUD_0_W+1), 0),
               G_HEALTHHUD_0_W, G_HEALTHHUD_0_H, g_tabletransp);
         }
         else {
            cpct_drawSpriteMaskedAlignedTable(g_healthhud_0,
               cpct_getScreenPtr(HEALTH_DISPLAY_VMEM, i*(G_HEALTHHUD_0_W+1), 0),
               G_HEALTHHUD_0_W, G_HEALTHHUD_0_H, g_tabletransp);
         }
      }
      cpct_setBorder(HW_BLACK);      // (Re)set to black from red.
   }
   // Score HUD.
   if ((redrawHudFlags & REDRAW_HUD_SCORE) == REDRAW_HUD_SCORE) {
      char buffer[5];
      u16ToString(buffer, globalScore+levelScore);
      drawText(buffer, SCORE_DISPLAY_X+SCORE_DISPLAY_SIZE, SCORE_DISPLAY_Y, 0);
   }

   // Reset flags.
   redrawHudFlags = REDRAW_HUD_NOTHING;
}


/**
 * Renders a complete screen (map, entities).
 */
void renderScreenComplete() {
   cpct_etm_drawTilemap2x4_f(MAP_WIDTH, MAP_HEIGHT, MAP_POS, currentScreenTiles);

   // Render coins.
   for (i8 i = currentScreen->numCoins-1; i >= 0; i--) {
      TCoin *c = currentScreen->coins[i];
      if (c->entity.deletionState == DEL_ACTIVE) {
         renderSprite(c->entity.x, c->entity.y, &c->sprite);
      }
   }
   // Render keys.
   for (i8 i = currentScreen->numKeys-1; i >= 0; i--) {
      TKey *k = currentScreen->keys[i];
      if (k->entity.deletionState == DEL_ACTIVE) {
         renderSprite(k->entity.x, k->entity.y, &k->sprite);
      }
   }
   // Render locks.
   for (i8 i = currentScreen->numLocks-1; i >= 0; i--) {
      TLock *l = currentScreen->locks[i];
      renderSprite(l->entity.x, l->entity.y, &l->sprite);
   }
   // Render enemies.
   for (i8 i = currentScreen->numEnemies-1; i >= 0; i--) {
      TCharacter *e = currentScreen->enemies[i];
      if (e->entity.deletionState == DEL_ACTIVE) {
         eraseCharacter(e);
         renderAnimFrame(e);
      }
   }

   eraseCharacter(&hero);
   renderAnimFrame(&hero);
   if (finish.exists && finish.screenX == currentScreenX && finish.screenY == currentScreenY) {
      renderSprite(finish.entity.x, finish.entity.y, &finish.sprite);
   }

   // Hud.
   drawHud();
}


/**
 * Waits for vsync amd renders only sprites that need to be rendered.
 * Entities that are marked for deletion will be erased and marked as inactive.
 * Altough the game doesn't run at 50Hz, the time used to render is minimized to mitigate
 * flickering (some of it, anyway) but still not having to use double buffering.
 * |> VSync
 * |  erase needed sprites
 * |  render needed sprites
 * |  game loop ( a few VSyncs happen here)
 * |  prepare animations etc.
 * |  wait for VSync
 * |  (repeat)
 */
void renderScreen() {
   const u8 finishRedrawNeeded = finish.exists && finish.sprite.needsRedraw;
   u8 heroRedrawState;
   // Update character animations.
   if (hero.state != ST_DEAD) {
      updateCharAnimation(&hero);
   }
   heroRedrawState = hero.animation.redrawState;

   cpct_waitVSYNC();

   if (finishRedrawNeeded) {
      // Erase finish.
      cpct_etm_drawTileBox2x4(finish.entity.x/2, finish.entity.x/4, G_FINISH_W/2, G_FINISH_H/4, MAP_WIDTH, MAP_POS, currentScreenTiles);
   }

   // Erase enemies.
   for (i8 i = currentScreen->numEnemies-1; i >= 0; i--) {
      TCharacter *e = currentScreen->enemies[i];
      u8 delState = e->entity.deletionState;
      if (delState == DEL_ACTIVE) {
         updateCharAnimation(e);
         if (e->animation.redrawState) {
            eraseCharacter(e);
         }
      }
      else if (delState == DEL_QUEUED) {
         eraseCharacter(e);
         e->entity.deletionState = DEL_INACTIVE;
      }
   }
   // Erase hero.
   if (heroRedrawState) {
      eraseCharacter(&hero);
   }
   // Erase & render coins.
   for (i8 i = currentScreen->numCoins-1; i >= 0; i--) {
      TCoin *c = currentScreen->coins[i];
      if (c->entity.deletionState == DEL_ACTIVE) {
         if (c->sprite.needsRedraw) {
            renderSprite(c->entity.x, c->entity.y, &c->sprite);
            c->sprite.needsRedraw = FALSE;
         }
      }
      else if (c->entity.deletionState == DEL_QUEUED) {
         eraseCoin(c);
         c->entity.deletionState = DEL_INACTIVE;
      }

   }
   // Erase & render keys.
   for (i8 i = currentScreen->numKeys-1; i >= 0; i--) {
      TKey *k = currentScreen->keys[i];
      if (k->entity.deletionState == DEL_ACTIVE) {
         if (k->sprite.needsRedraw) {
            renderSprite(k->entity.x, k->entity.y, &k->sprite);
            k->sprite.needsRedraw = FALSE;
         }
      }
      else if (k->entity.deletionState == DEL_QUEUED) {
         eraseKey(k);
         k->entity.deletionState = DEL_INACTIVE;
      }
   }
   // Erase & render locks.
   for (i8 i = currentScreen->numLocks-1; i >= 0; i--) {
      TLock *l = currentScreen->locks[i];
      if (l->sprite.needsRedraw) {
         renderSprite(l->entity.x, l->entity.y, &l->sprite);
         l->sprite.needsRedraw = FALSE;
      }
   }
   // Render enemies.
   for (i8 i = currentScreen->numEnemies-1; i >= 0; i--) {
      TCharacter *e = currentScreen->enemies[i];
      if (e->entity.deletionState == DEL_ACTIVE && e->animation.redrawState) {
         renderCharacter(e);
      }
   }

   if (finishRedrawNeeded) {
      renderSprite(finish.entity.x, finish.entity.y, &finish.sprite);
      finish.sprite.needsRedraw = FALSE;
   }
   if (heroRedrawState) {
      renderCharacter(&hero);
   }

   drawHud();
}


/**
 *
 * @param string Char arrary, null terminated.
 * @param posX
 * @param posY
 * @param transparent If false, draws a black box behind text (background color 0 would be pink otherwise)
 */
void drawText(char string[], u8 posX, u8 posY, u8 transparent) {
   u8 *character = string, *pvmem, index, *sprite;

   pvmem = cpct_getScreenPtr(CPCT_VMEM_START, posX, posY);
   while (*character) {     // Loop until \0 terminator.
      index = (*character++) - 32;
      sprite = g_font_tileset[index];
      if (transparent) {
         cpct_drawSpriteMaskedAlignedTable(sprite, pvmem, G_FONT_BITMAPFONT_00_W, G_FONT_BITMAPFONT_00_H, g_tabletransp);
      }
      else {
         cpct_drawSolidBox(pvmem, PATTERN_BLACK, G_FONT_BITMAPFONT_00_W, G_FONT_BITMAPFONT_00_H);
         cpct_drawSpriteMaskedAlignedTable(sprite, pvmem, G_FONT_BITMAPFONT_00_W, G_FONT_BITMAPFONT_00_H, g_tabletransp);
      }
      pvmem += G_FONT_BITMAPFONT_00_W;
   }
}