#include "utils.h"

#define BASE 10

u8 u8ToChars(char* buffer, u8 value, u8 idx) {
   u8 new = value / BASE;
   if (new > 0) {
      idx = u8ToChars(buffer, new, idx);
   }
   buffer[idx++] = '0' + (value % BASE);
   return idx;
}


/**
 * Convert decimal number to string.
 * @param buffer
 * @param value
 */
void u8ToString(char* buffer, u8 value) {
   u8 idx;
   idx = u8ToChars(buffer, value, 0);
   buffer[idx] = '\0';
}


u8 u16ToChars(char* buffer, u16 value, u8 idx) {
   u16 new = value / BASE;
   if (new > 0) {
      idx = u16ToChars(buffer, new, idx);
   }
   buffer[idx++] = '0' + (value % BASE);
   return idx;
}


/**
 * Convert decimal number to string.
 * @param buffer
 * @param value
 */
void u16ToString(char* buffer, u16 value) {
   u8 idx;
   idx = u16ToChars(buffer, value, 0);
   buffer[idx] = '\0';
}


/**
 * Fast collision check between AABB.
 * @param x1
 * @param y1
 * @param w1
 * @param h1
 * @param x2
 * @param y2
 * @param w2
 * @param h2
 * @return 1 when the two rectangles collide, otherwise 0.
 */
inline u8 isColliding(u8 x1, u8 y1, u8 w1, u8 h1, u8 x2, u8 y2, u8 w2, u8 h2) {
   return !(   (x1 + w1 < x2 || x1 > x2 + w2)      // Missing on X axis or
            || (y1 + h1 < y2 || y1 > y2 + h2));    // Missing on Y axis.
}


/**
 * The squared distance can never be greater than 240 in our 80x160 game world.
 * @param x1
 * @param y1
 * @param x2
 * @param y2
 * @return The distance between two points, squared.
 */
inline u8 calcManhattanDistance(u8 x1, u8 y1, u8 x2, u8 y2) {
   i16 dX = x1 - x2;    // max dist 80, so one signed byte is enough.
   i16 dY = y1 - y2;    // max dist 160, so two signed bytes needed.
   return   (dX < 0 ? -dX : dX)
          + (dY < 0 ? -dY : dY);   // Calculate absolute distance.
}